use mongodb::{Collection, bson::doc};

#[derive(serde::Deserialize, serde::Serialize)]
pub struct ItemEntry {
    pub name: String,
    pub category: String
}

impl ItemEntry {
    pub fn new(doc: mdq::Document) -> Self {
        let name = std::path::Path::new(&doc.path).file_stem().unwrap().to_str().unwrap().to_string();
        let category = doc.frontmatter.as_mapping().unwrap().get("category").unwrap().as_str().unwrap().to_string();
        Self { name, category }
    }
}

pub struct ItemDB {
    index: mdq::Index,
    mongodb: mongodb::Client
}

impl ItemDB {
    pub async fn new(dir: &str, mongodb: &str) -> Self {
        // scan for markdown item entries
        let index = mdq::Index::new(dir, true);
        let mongodb = mongodb::Client::with_uri_str(mongodb).await.unwrap();

        for item in &index.documents {
            let item = ItemEntry::new(item.clone());
            log::info!("Adding item {} to DB", item.name);
            let items: Collection<ItemEntry> = mongodb.database("cdb").collection("items");
            items.insert_one(item, None).await.unwrap();
        }

        Self { index, mongodb }
    }

    pub fn get_item(&self, item: &str) -> Option<ItemEntry> {
        self.index.documents.iter().map(|x| ItemEntry::new(x.clone())).find(|x| x.name == item)
    }

    pub fn items(&self) -> Vec<String> {
        let mut ret = vec![];
        for item in &self.index.documents {
            let item = ItemEntry::new(item.clone());
            ret.push(item.name);
        }
        ret
    }
}